PFont font;

int currentClass = -1;
float minDistance = 1e10;

void setup() {
  size(1200, 680, "processing.core.PGraphicsRetina2D");
  background(255);
  smooth();
  font = createFont("Helvetica", 16, true);
}

void drawSOMMap() {  
  float cellWidth = 7;
  float cellHeight = 7;
  float cellMargin = 1;
  
  // Title
  textAlign(LEFT, CENTER);
  textFont(font, 16);
  noStroke();
  fill(0, 0, 0);
  text("SOM Map of Dewey Class Temporal Trends", 0, 0);
  
  translate(0, 30);
  
  int mouseX_translated = mouseX - 50;
  int mouseY_translated = mouseY - 50;
  
  // Draw the SOM grid.
  for(int i = 0; i < somWidth; i++) {
    for(int j = 0; j < somHeight; j++) {
      noStroke();
      fill(245, 245, 245);
      rect(i * (cellWidth + cellMargin), j * (cellHeight + cellMargin), cellWidth, cellHeight);
    }
  }
  
  float maxNumBibs = 0;
  for(int i = 0; i < deweySOM.length; i++) {
    int numBibs = deweySOM[i][3];
    if(maxNumBibs < numBibs) maxNumBibs = numBibs;
  }
  
  for(int i = 0; i < deweySOM.length; i++) {
    int deweyClass = deweySOM[i][0];
    int x = deweySOM[i][1];
    int y = deweySOM[i][2];
    int numBibs = deweySOM[i][3];
    if(numBibs <= 0) continue; 
    deweyStrokeColor(deweyClass, 255);
    float cx = x * (cellWidth + cellMargin) + cellWidth / 2;
    float cy = y * (cellHeight + cellMargin)+ cellHeight / 2;
    float scale = log(1 + numBibs) / log(1 + maxNumBibs);
    float w = scale * 40;
    deweyColor(deweyClass, 64);
    //fill(31, 119, 180, 64);
    
    float distance = sqrt((cx - mouseX_translated) * (cx - mouseX_translated) + (cy - mouseY_translated) * (cy - mouseY_translated));
    if(distance < w && distance < minDistance) {
      currentClass = deweyClass;
      minDistance = distance;
    }
    //ellipse(cx, cy, w, w);
    rect(cx - w / 2.0, cy - w / 2.0, w, w);
    textAlign(CENTER, CENTER);
    textFont(font, 10);
    noStroke(); fill(0, 0, 0);
    //text(deweyClass, cx, cy - 1);
  }
  if(currentClass >= 0) {
    int i = currentClass;
    int deweyClass = deweySOM[i][0];
    int x = deweySOM[i][1];
    int y = deweySOM[i][2];
    int numBibs = deweySOM[i][3];
    stroke(255, 127, 14);
    float cx = x * (cellWidth + cellMargin) + cellWidth / 2;
    float cy = y * (cellHeight + cellMargin)+ cellHeight / 2;
    float scale = log(1 + numBibs) / log(1 + maxNumBibs);
    float w = scale * 40;
    fill(255, 127, 14, 255);
    //ellipse(cx, cy, w, w);
    rect(cx - w / 2.0, cy - w / 2.0, w, w);
    textAlign(CENTER, CENTER);
    textFont(font, 10);
    noStroke(); fill(255, 255, 255, 255);
    text(deweyClass, cx, cy - 1);
  }
}

void drawRBMMap() {
  float map_width = 399;
  float map_height = 399;
  
  // Title
  textAlign(LEFT, CENTER);
  textFont(font, 16);
  noStroke();
  fill(0, 0, 0);
  text("RBM+PCA Map of Dewey Class Temporal Trends", 0, 0);
  
  translate(0, 30);
  
  noStroke();
  fill(245, 245, 245);
  rect(0, 0, map_width, map_height);
  
  int mouseX_translated = mouseX - 500;
  int mouseY_translated = mouseY - 50;
  
  float maxNumBibs = 0;
  for(int i = 0; i < deweySOM.length; i++) {
    int numBibs = deweySOM[i][3];
    if(maxNumBibs < numBibs) maxNumBibs = numBibs;
  }
  
  for(int i = 0; i < deweySOM.length; i++) {
    int deweyClass = (int)deweyRBM[i][0];
    float x = (1 + deweyRBM[i][1]) / 2;
    float y = (1 + deweyRBM[i][2]) / 2;
    int numBibs = (int)deweyRBM[i][3];
    if(numBibs <= 0) continue; 
    deweyStrokeColor(deweyClass, 255);
    float cx = x * map_width;
    float cy = y * map_height;
    float scale = log(1 + numBibs) / log(1 + maxNumBibs);
    float w = scale * 30;
    deweyColor(deweyClass, 64);
    //fill(31, 119, 180, 32);
    //ellipse(cx, cy, w, w);
    float distance = sqrt((cx - mouseX_translated) * (cx - mouseX_translated) + (cy - mouseY_translated) * (cy - mouseY_translated));
    if(distance < w && distance < minDistance) {
      currentClass = deweyClass;
      minDistance = distance;
    }
    rect(cx - w / 2.0, cy - w / 2.0, w, w);
    textAlign(CENTER, CENTER);
    textFont(font, 8);
    noStroke();
    fill(0, 0, 0);
    //text(deweyClass, cx, cy - 1);
  }
  if(currentClass >= 0) {
    int i = currentClass;
    int deweyClass = (int)deweyRBM[i][0];
    float x = (1 + deweyRBM[i][1]) / 2;
    float y = (1 + deweyRBM[i][2]) / 2;
    int numBibs = (int)deweyRBM[i][3];
    stroke(255, 127, 14);
    float cx = x * map_width;
    float cy = y * map_height;
    float scale = log(1 + numBibs) / log(1 + maxNumBibs);
    float w = scale * 30;
    fill(255, 127, 14, 255);
    //ellipse(cx, cy, w, w);
    rect(cx - w / 2.0, cy - w / 2.0, w, w);
    textAlign(CENTER, CENTER);
    textFont(font, 8);
    noStroke();
    fill(255, 255, 255, 255);
    text(deweyClass, cx, cy - 1);
  }
}

void drawTimeline() {
  float timeline_width = 849;
  float timeline_height = 100;
  textAlign(LEFT, CENTER);
  textFont(font, 16);
  noStroke();
  fill(0);
  
  if(currentClass >= 0) {
    text("Timelines (" + classnames[currentClass] + ")", 0, 0);
  } else {  
    text("Timelines", 0, 0);
  }
  
  translate(0, 30);
 
  for(int j = 0; j < deweyTimeline[0].length; j++) {
    float x = ((float)j + 0.5) / (deweyTimeline[0].length) * timeline_width;
    float w = 1.0 / deweyTimeline[0].length * timeline_width - 0.5;
    fill(245); noStroke();
    rect(x - w / 2, 0, w, timeline_height);
    rect(x - w / 2, timeline_height + 1, w, 12);
    fill(0); noStroke();
    textAlign(CENTER, TOP);
    textFont(font, 9);
    text((j) % 12 + 1, x, timeline_height + 3);
  }
  for(int year = 0; year < 5; year++) {
    float x = ((float)(year * 12 + 5.5) + 0.5) / (deweyTimeline[0].length) * timeline_width;
    float w = 12.0 / deweyTimeline[0].length * timeline_width - 0.5;
    fill(245); noStroke();
    rect(x - w / 2, timeline_height + 14, w, 14);
    fill(0); noStroke();
    textAlign(CENTER, TOP);
    textFont(font, 11);
    text(year + 2010, x, timeline_height + 16);
  }
  // Draw timelines.
  for(int i = 0; i < deweyTimeline.length; i++) {
     noFill();
     deweyStrokeColor(i, 32);
     //stroke(31, 119, 180, 32);
     beginShape();
     for(int j = 0; j < deweyTimeline[i].length; j++) {
       float x = ((float)j + 0.5) / (deweyTimeline[i].length) * timeline_width;
       float y = (1.0 - deweyTimeline[i][j] / 3) * timeline_height;
       vertex(x, y);
     }
     endShape();
  }
  if(currentClass >= 0) {
     noFill();
     stroke(255, 127, 14, 255);
     beginShape();
     int i = currentClass;
     for(int j = 0; j < deweyTimeline[i].length; j++) {
       float x = ((float)j + 0.5) / (deweyTimeline[i].length) * timeline_width;
       float y = (1.0 - deweyTimeline[i][j] / 3.0) * timeline_height;
       vertex(x, y);
     }
     endShape();
  }
}

void drawLegend() {
  for(int i = 0; i < 10; i++) {
    pushMatrix();
    textFont(font, 11);
    textAlign(LEFT, CENTER);
    deweyColor(i * 10, 255);
    text(classnames[i * 10], 0, 0);
    translate(5, 15);
    for(int j = 0; j < 10; j++) {
      int cls = i * 10 + j;
      if(cls == currentClass) {
          stroke(255, 127, 14, 255);
          fill(255, 127, 14, 255);
      } else {
          deweyStrokeColor(i * 10 + j, 255);
          deweyColor(i * 10 + j, 64);
      }
      rect(-5, -5, 10, 10);
      
      float mx = mouseX - 920 - 5 - 12 * j;
      float my = mouseY - 50 - 31 * i - 15;
      if(abs(mx) < 5 && abs(my) < 5) currentClass = cls;
      
      translate(12, 0);
    }
    popMatrix();
    translate(0, 31);
  }
}

void draw() {
  background(255, 255, 255, 255);

  minDistance = 1e10;
  pushMatrix();
  translate(50, 20);
  drawSOMMap();
  popMatrix();
  
  pushMatrix();
  translate(500, 20);
  drawRBMMap();
  popMatrix();
  
  pushMatrix();
  translate(50, 490);
  drawTimeline();
  popMatrix();
  
  pushMatrix();
  translate(920, 50);
  drawLegend();
  popMatrix();
}
